/*
/ Dialogs.cpp
/ misc dialogs
/
/ version 1.0, 2012 June 8
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2012  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU General Public License for more details.
/
/    You should have received a copy of the GNU General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#include "Classdef.h"

MyTipWindow::MyTipWindow(MyMapView * parent, wxString & text, int x,
                         int y, int category):wxWindow(parent, wxID_ANY,
                                                       wxDefaultPosition,
                                                       wxDefaultSize,
                                                       wxBORDER_NONE)
{
  int xx = x + 5;
  int yy = y + 5;
  Parent = parent;
  Category = category;
  switch (Category)
    {
      case ATLAS_TOWNS_MINOR:
      case ATLAS_TOWNS_MEDIUM:
      case ATLAS_TOWNS_MAJOR:
      case ATLAS_TOWN_AREAS_MINOR:
      case ATLAS_TOWN_AREAS_MEDIUM:
      case ATLAS_TOWN_AREAS_MAJOR:
        Text = wxT("Town: ") + text;
        break;
      case ATLAS_RIVERS:
        Text = wxT("River: ") + text;
        break;
      case ATLAS_LAKES:
        Text = wxT("Lake: ") + text;
        break;
      case ATLAS_NATURAL_RESERVES:
        Text = wxT("Natural: ") + text;
        break;
      case ATLAS_RAILWAYS:
        Text = wxT("Railway: ") + text;
        break;
      case ATLAS_RAILWAY_STATIONS:
        Text = wxT("Station: ") + text;
        break;
      case ATLAS_CULTURAL_HERITAGE:
        Text = wxT("Cultural: ") + text;
        break;
      case ATLAS_HIGHWAYS:
        Text = wxT("Road: ") + text;
        break;
      case ATLAS_COUNTRIES:
        Text = wxT("Country: ") + text;
        break;
      case ATLAS_ADMIN_AREAS_1:
      case ATLAS_ADMIN_AREAS_2:
      case ATLAS_ADMIN_AREAS_3:
        Text = wxT("Admin: ") + text;
        break;
      default:
        Text = text;
        break;
    };
  MouseX = x;
  MouseY = y;
// computing the text extent
  wxClientDC dc(this);
  dc.SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
  wxSize sz = dc.GetTextExtent(Text);
  wxSize szParent = Parent->GetClientSize();
  if ((sz.GetWidth() + 16 + MouseX) >= szParent.GetWidth())
    xx = MouseX - sz.GetWidth() - 15;
  if ((sz.GetHeight() + 16 + MouseY) >= szParent.GetHeight())
    yy = MouseY - sz.GetHeight() - 15;
  SetSize(xx, yy, sz.GetWidth() + 10, sz.GetHeight() + 10);
  Show();
// connecting event handlers
  Connect(wxID_ANY, wxEVT_PAINT,
          (wxObjectEventFunction) & MyTipWindow::OnPaint);
  Connect(wxID_ANY, wxEVT_ERASE_BACKGROUND,
          (wxObjectEventFunction) & MyTipWindow::OnEraseBackground);
  Connect(wxID_ANY, wxEVT_MOTION,
          wxMouseEventHandler(MyTipWindow::OnMouseEvent), NULL, this);
}

bool MyTipWindow::IsToBeErased(int x, int y)
{
  if (abs(x - MouseX) > 4)
    return true;
  if (abs(y - MouseY) > 4)
    return true;
  return false;
}

void MyTipWindow::OnPaint(wxPaintEvent & WXUNUSED(event))
{
  wxPaintDC dc(this);
  wxSize sz = GetClientSize();
  switch (Category)
    {
      case ATLAS_TOWNS_MINOR:
      case ATLAS_TOWNS_MEDIUM:
      case ATLAS_TOWNS_MAJOR:
      case ATLAS_TOWN_AREAS_MAJOR:
      case ATLAS_TOWN_AREAS_MINOR:
      case ATLAS_TOWN_AREAS_MEDIUM:
        dc.SetBrush(wxBrush(wxColour(0, 0, 0)));
        dc.SetPen(wxPen(wxColour(192, 192, 192)));
        break;
      case ATLAS_LAND_AREAS:
      case ATLAS_MARINE_AREAS:
        dc.SetBrush(wxBrush(wxColour(204, 204, 204)));
        dc.SetPen(wxPen(wxColour(0, 0, 0)));
        break;
      case ATLAS_LAKES:
      case ATLAS_RIVERS:
        dc.SetBrush(wxBrush(wxColour(204, 204, 255)));
        dc.SetPen(wxPen(wxColour(0, 0, 0)));
        break;
      case ATLAS_NATURAL_RESERVES:
        dc.SetBrush(wxBrush(wxColour(204, 255, 204)));
        dc.SetPen(wxPen(wxColour(0, 0, 0)));
        break;
      default:
        dc.SetBrush(wxBrush(wxColour(255, 255, 255)));
        dc.SetPen(wxPen(wxColour(0, 0, 0)));
        break;
    };
  dc.DrawRectangle(0, 0, sz.GetWidth(), sz.GetHeight());
  switch (Category)
    {
      case ATLAS_TOWNS_MINOR:
      case ATLAS_TOWNS_MEDIUM:
      case ATLAS_TOWNS_MAJOR:
      case ATLAS_TOWN_AREAS_MAJOR:
      case ATLAS_TOWN_AREAS_MINOR:
      case ATLAS_TOWN_AREAS_MEDIUM:
        dc.SetTextBackground(wxColour(0, 0, 0));
        dc.SetTextForeground(wxColour(255, 255, 255));
        break;
      case ATLAS_LAND_AREAS:
      case ATLAS_MARINE_AREAS:
        dc.SetTextBackground(wxColour(204, 204, 204));
        dc.SetTextForeground(wxColour(0, 0, 0));
        break;
      case ATLAS_LAKES:
      case ATLAS_RIVERS:
        dc.SetTextBackground(wxColour(204, 204, 255));
        dc.SetTextForeground(wxColour(0, 0, 0));
        break;
      default:
        dc.SetTextBackground(wxColour(255, 255, 192));
        dc.SetTextForeground(wxColour(0, 0, 0));
        break;
    };
  dc.SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
  dc.DrawText(Text, 5, 5);
}

void MyTipWindow::OnMouseEvent(wxMouseEvent & event)
{
//
// MOUSE motion - event handler
// 
  event.Skip();
}

CountryInfoDialog::CountryInfoDialog(MyMapView * parent,
                                     wxString & continent_code,
                                     wxString & continent_name,
                                     wxString & country_code,
                                     wxString & country_name,
                                     wxString & capital,
                                     wxString & currency_code,
                                     wxString & currency_name, double area_km2,
                                     int population, double pop_km2,
                                     wxBitmap & flag, wxBitmap & emblem)
{
//
// constructor
//
  char dummy[80];
  Parent = parent;
  Continent = continent_name;
  if (continent_code.Len() > 0)
    Continent += wxT(" (") + continent_code + wxT(")");
  Country = country_name;
  if (country_code.Len() > 0)
    Country += wxT(" (") + country_code + wxT(")");
  Capital = capital;
  Currency = currency_name;
  if (currency_code.Len() > 0)
    Currency += wxT(" (") + currency_code + wxT(")");
  if (area_km2 > 0.0)
    {
      sprintf(dummy, "%1.2f sq.Km", area_km2);
      AreaKm2 = wxString::FromUTF8(dummy);
    }
  if (population > 0)
    {
      sprintf(dummy, "%d", population);
      Population = wxString::FromUTF8(dummy);
    }
  if (pop_km2 > 0.0)
    {
      sprintf(dummy, "%1.2f persons per sq.Km", pop_km2);
      PopKm2 = wxString::FromUTF8(dummy);
    }
  Flag = flag;
  Emblem = emblem;
  if (wxDialog::Create(Parent, wxID_ANY, wxT("LibreAtlas")) == false)
    return;
// populates individual controls
  CreateControls();
// sets dialog sizer
  GetSizer()->Fit(this);
  GetSizer()->SetSizeHints(this);
// centers the dialog window
  Centre();
}

void CountryInfoDialog::CreateControls()
{
//
// creating individual controls and setting initial values
//
  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);
  this->SetSizer(topSizer);
  wxBoxSizer *boxSizer = new wxBoxSizer(wxVERTICAL);
  topSizer->Add(boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
  wxBoxSizer *grpSizer = new wxBoxSizer(wxHORIZONTAL);
  boxSizer->Add(grpSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
  wxBoxSizer *leftSizer = new wxBoxSizer(wxVERTICAL);
  grpSizer->Add(leftSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
  wxBoxSizer *rightSizer = new wxBoxSizer(wxVERTICAL);
  grpSizer->Add(rightSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);

// first row: continent
  wxStaticText *contLabel =
    new wxStaticText(this, wxID_STATIC, wxT("Continent"), wxDefaultPosition,
                     wxSize(70, 22));
  leftSizer->Add(contLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
  ContinentCtrl =
    new wxTextCtrl(this, wxID_ANY, Continent, wxDefaultPosition,
                   wxSize(350, 22), wxTE_READONLY);
  rightSizer->Add(ContinentCtrl, 0, wxALIGN_RIGHT | wxALL, 2);

// second row: country
  wxStaticText *countryLabel =
    new wxStaticText(this, wxID_STATIC, wxT("Country"), wxDefaultPosition,
                     wxSize(70, 22));
  leftSizer->Add(countryLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
  CountryCtrl =
    new wxTextCtrl(this, wxID_ANY, Country, wxDefaultPosition,
                   wxSize(350, 22), wxTE_READONLY);
  rightSizer->Add(CountryCtrl, 0, wxALIGN_RIGHT | wxALL, 2);

// third row: capital
  wxStaticText *capLabel =
    new wxStaticText(this, wxID_STATIC, wxT("Capital"), wxDefaultPosition,
                     wxSize(70, 22));
  leftSizer->Add(capLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
  CapitalCtrl =
    new wxTextCtrl(this, wxID_ANY, Capital, wxDefaultPosition,
                   wxSize(350, 22), wxTE_READONLY);
  rightSizer->Add(CapitalCtrl, 0, wxALIGN_RIGHT | wxALL, 2);

// fourth row: currency
  wxStaticText *currLabel =
    new wxStaticText(this, wxID_STATIC, wxT("Currency"), wxDefaultPosition,
                     wxSize(70, 22));
  leftSizer->Add(currLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
  CurrencyCtrl =
    new wxTextCtrl(this, wxID_ANY, Currency, wxDefaultPosition,
                   wxSize(350, 22), wxTE_READONLY);
  rightSizer->Add(CurrencyCtrl, 0, wxALIGN_RIGHT | wxALL, 2);

// fifth row: population
  wxStaticText *popLabel =
    new wxStaticText(this, wxID_STATIC, wxT("Population"), wxDefaultPosition,
                     wxSize(70, 22));
  leftSizer->Add(popLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
  PopulationCtrl =
    new wxTextCtrl(this, wxID_ANY, Population, wxDefaultPosition,
                   wxSize(350, 22), wxTE_READONLY);
  rightSizer->Add(PopulationCtrl, 0, wxALIGN_RIGHT | wxALL, 2);

// sixth row: AreaKm2
  wxStaticText *areaLabel =
    new wxStaticText(this, wxID_STATIC, wxT("Area"), wxDefaultPosition,
                     wxSize(70, 22));
  leftSizer->Add(areaLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
  AreaKm2Ctrl =
    new wxTextCtrl(this, wxID_ANY, AreaKm2, wxDefaultPosition,
                   wxSize(350, 22), wxTE_READONLY);
  rightSizer->Add(AreaKm2Ctrl, 0, wxALIGN_RIGHT | wxALL, 2);

// seventh row: popKm2
  wxStaticText *densLabel =
    new wxStaticText(this, wxID_STATIC, wxT("Density"), wxDefaultPosition,
                     wxSize(70, 22));
  leftSizer->Add(densLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
  PopKm2Ctrl =
    new wxTextCtrl(this, wxID_ANY, PopKm2, wxDefaultPosition,
                   wxSize(350, 22), wxTE_READONLY);
  rightSizer->Add(PopKm2Ctrl, 0, wxALIGN_RIGHT | wxALL, 2);

// Flag and Emblem
  wxBoxSizer *imgSizer = new wxBoxSizer(wxHORIZONTAL);
  boxSizer->Add(imgSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
  wxStaticBitmap *FlagCtrl = new wxStaticBitmap(this, wxID_STATIC,
                                                Flag, wxDefaultPosition,
                                                wxDefaultSize, wxBORDER_SIMPLE);
  imgSizer->Add(FlagCtrl, 0, wxALIGN_LEFT | wxALL, 2);
  wxStaticBitmap *EmblemCtrl = new wxStaticBitmap(this, wxID_STATIC,
                                                  Emblem, wxDefaultPosition,
                                                  wxDefaultSize,
                                                  wxBORDER_SIMPLE);
  imgSizer->Add(EmblemCtrl, 0, wxALIGN_RIGHT | wxALL, 2);

// OK button
  wxBoxSizer *okBox = new wxBoxSizer(wxHORIZONTAL);
  boxSizer->Add(okBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
  wxButton *ok = new wxButton(this, wxID_OK, wxT("&OK"));
  okBox->Add(ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
  Connect(wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
          (wxObjectEventFunction) & CountryInfoDialog::OnOk);
}

void CountryInfoDialog::OnOk(wxCommandEvent & WXUNUSED(event))
{
//
// all done: 
//
  wxDialog::EndModal(wxID_OK);
}

AdminAreaInfoDialog::AdminAreaInfoDialog(MyMapView * parent,
                                         wxString & name,
                                         wxString & type,
                                         wxBitmap & coat_of_arms)
{
//
// constructor
//
  Parent = parent;
  Name = name;
  Type = type;
  CoatOfArms = coat_of_arms;
  if (wxDialog::Create(Parent, wxID_ANY, wxT("LibreAtlas")) == false)
    return;
// populates individual controls
  CreateControls();
// sets dialog sizer
  GetSizer()->Fit(this);
  GetSizer()->SetSizeHints(this);
// centers the dialog window
  Centre();
}

void AdminAreaInfoDialog::CreateControls()
{
//
// creating individual controls and setting initial values
//
  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);
  this->SetSizer(topSizer);
  wxBoxSizer *boxSizer = new wxBoxSizer(wxVERTICAL);
  topSizer->Add(boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
  wxBoxSizer *grpSizer = new wxBoxSizer(wxHORIZONTAL);
  boxSizer->Add(grpSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
  wxBoxSizer *leftSizer = new wxBoxSizer(wxVERTICAL);
  grpSizer->Add(leftSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
  wxBoxSizer *rightSizer = new wxBoxSizer(wxVERTICAL);
  grpSizer->Add(rightSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);

// first row: name
  wxStaticText *nameLabel =
    new wxStaticText(this, wxID_STATIC, wxT("Name"), wxDefaultPosition,
                     wxSize(70, 22));
  leftSizer->Add(nameLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
  NameCtrl =
    new wxTextCtrl(this, wxID_ANY, Name, wxDefaultPosition,
                   wxSize(350, 22), wxTE_READONLY);
  rightSizer->Add(NameCtrl, 0, wxALIGN_RIGHT | wxALL, 2);

// second row: admin-level
  wxStaticText *levelLabel =
    new wxStaticText(this, wxID_STATIC, wxT("Admin-Level"), wxDefaultPosition,
                     wxSize(70, 22));
  leftSizer->Add(levelLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
  TypeCtrl =
    new wxTextCtrl(this, wxID_ANY, Type, wxDefaultPosition,
                   wxSize(350, 22), wxTE_READONLY);
  rightSizer->Add(TypeCtrl, 0, wxALIGN_RIGHT | wxALL, 2);

// Coat of Arms
  wxBoxSizer *imgSizer = new wxBoxSizer(wxHORIZONTAL);
  boxSizer->Add(imgSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
  wxStaticBitmap *CoatOfArmsCtrl = new wxStaticBitmap(this, wxID_STATIC,
                                                      CoatOfArms,
                                                      wxDefaultPosition,
                                                      wxDefaultSize,
                                                      wxBORDER_SIMPLE);
  imgSizer->Add(CoatOfArmsCtrl, 0, wxALIGN_LEFT | wxALL, 2);

// OK button
  wxBoxSizer *okBox = new wxBoxSizer(wxHORIZONTAL);
  boxSizer->Add(okBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
  wxButton *ok = new wxButton(this, wxID_OK, wxT("&OK"));
  okBox->Add(ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
  Connect(wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
          (wxObjectEventFunction) & AdminAreaInfoDialog::OnOk);
}

void AdminAreaInfoDialog::OnOk(wxCommandEvent & WXUNUSED(event))
{
//
// all done: 
//
  wxDialog::EndModal(wxID_OK);
}
